/*****************************************************************************
 *
 * FILE:	check.c
 * DESCRIPTION:	check string utilities for simple CGI library
 * DATE:	Thu, Jun 20 2002
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.com/cover.html
 * COPYRIGHT:	(C) 2002 졿Kouichi ABE (WALL), All rights reserved.
 * $Id: check.c,v 1.5 2002/07/22 14:43:28 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#if	HAVE_UNISTD_H
#include <unistd.h>
#endif	/* HAVE_UNISTD_H */
#include "cockatrice.h"
#include "functions.h"


/*****************************************************************************
 *
 *	definition of macros and structures
 *
 *****************************************************************************/


/*****************************************************************************
 *
 *	declaration of local variables
 *
 *****************************************************************************/


/*****************************************************************************
 *
 *	definition of functions
 *
 *****************************************************************************/

/*
 * Function Name:	checkEmpty
 * Description:		check string with empty
 * Arguments:		string
 * Return Value:	Empty: 0, otherwise: -1
 */
int
checkEmpty(s)
	const String	s;
{
  return (s == NULL || s[0] == EOL) ? 0 : -1;
}

/*
 * Function Name:	checkDigit
 * Description:		check string with digit
 * Arguments:		string
 * Return Value:	Digit: 0, otherwise: -1
 */
int
checkDigit(s)
	const String	s;
{
  register int	i;

  if (s == NULL || s[0] == EOL) {
    return -1;
  }

  for (i = 0; s[i] != EOL; i++) {
    if (s[i] < '0' || s[i] > '9') {
      return -1;
    }
  }
  return 0;
}

/*
 * Function Name:	checkAlphabet
 * Description:		check string with alphabet
 * Arguments:		string
 * Return Value:	Alphabet: 0, otherwise: -1
 */
int
checkAlphabet(s)
	const String	s;
{
  register int	i;

  if (s == NULL || s[0] == EOL) {
    return -1;
  }

  for (i = 0; s[i] != EOL; i++) {
    if ((s[i] < 'A' || s[i] > 'Z') && (s[i] < 'a' || s[i] > 'z')) {
      return -1;
    }
  }
  return 0;
}

/*
 * Function Name:	checkUserName
 * Description:		check string with username
 * Arguments:		string
 * Return Value:	UserName: 0, otherwise: -1
 * NOTE:		'^[a-z][a-z0-9]+$'
 */
int
checkUserName(s)
	const String	s;
{
  register int	i;

  if (s == NULL || s[0] == EOL) {
    return -1;
  }

  if (s[0] < 'a' || s[0] > 'z') {
    return -1;
  }
  for (i = 1; s[i] != EOL; i++) {
    if ((s[i] < '0' || s[i] > '9') && (s[i] < 'a' || s[i] > 'z')) {
      return -1;
    }
  }
  return 0;
}

/*
 * Function Name:	checkHostName
 * Description:		check string with hostname
 * Arguments:		string
 * Return Value:	HostName: 0, otherwise: -1
 * NOTE:		'^[a-zA-Z][a-zA-Z0-9]+$'
 */
int
checkHostName(s)
	const String	s;
{
  register int	i;
  size_t	len;

  if (s == NULL || s[0] == EOL) {
    return -1;
  }

  if ((s[0] < 'A' || s[0] > 'Z') && (s[0] < 'a' || s[0] > 'z')) {
    return -1;
  }

  len = strlen(s);
  for (i = 1; i < len; i++) {
    if ((s[i] < '0' || s[i] > '9') &&
	(s[i] < 'A' || s[i] > 'Z') && (s[i] < 'a' || s[i] > 'z')) {
      return -1;
    }
  }
  return 0;
}

/*
 * Function Name:	checkDomainName
 * Description:		check string with DomainName
 * Arguments:		string
 * Return Value:	DomainName: 0, otherwise: -1
 */
int
checkDomainName(s)
	const String	s;
{
  register int	i;
  size_t	len;
  int		dot = 0;

  if (s == NULL || s[0] == EOL) {
    return -1;
  }

  if ((s[0] < 'A' || s[0] > 'Z') && (s[0] < 'a' || s[0] > 'z') &&
      (s[0] < '0' || s[0] > '9')) {
    return -1;
  }

  len = strlen(s) - 1;
  for (i = 1; i < len; i++) {
    if ((s[i] < 'A' || s[i] > 'Z') && (s[i] < 'a' || s[i] > 'z') &&
	(s[i] < '0' || s[i] > '9') && s[i] != '-' && s[i] != '.') {
      return -1;
    }
    if (s[i] == '.') {
      if (s[i - 1] == '-' || s[i + 1] == '-') {
	return -1;
      }
      dot++;
    }
  }
  if ((s[len] < 'A' || s[len] > 'Z') && (s[len] < 'a' || s[len] > 'z')) {
    return -1;
  }
  return dot > 0 ? 0 : -1;
}

/*
 * Function Name:	checkEMail
 * Description:		check string with EMail
 * Arguments:		string
 * Return Value:	EMail: 0, otherwise: -1
 */
int
checkEMail(str)
	const String	str;
{
  register int	i;
  size_t	len;
  String	s;	/* working buffer */
  String	domain;

  if (str == NULL || str[0] == EOL) {
    return -1;
  }

  s = strdup(str);
  if (s == NULL) {
    return -1;
  }

  domain = strchr(s, '@');
  if (domain == NULL) {
    return -1;
  }
  *domain++ = EOL;

  if (checkDomainName(domain) == -1) {
    return -1;
  }

  if ((s[0] < 'A' || s[0] > 'Z') && (s[0] < 'a' || s[0] > 'z') &&
      (s[0] < '0' || s[0] > '9')) {
    return -1;
  }

  len = strlen(s) - 1;
  for (i = 1; i < len; i++) {
    if ((s[i] < 'A' || s[i] > 'Z') && (s[i] < 'a' || s[i] > 'z') &&
	(s[i] < '0' || s[i] > '9') && s[i] != '-') {
      return -1;
    }
  }
  if ((s[len] < 'A' || s[len] > 'Z') && (s[len] < 'a' || s[len] > 'z') &&
      (s[len] < '0' || s[len] > '9')) {
    return -1;
  }
  return 0;
}

/*
 * Function Name:	checkRange
 * Description:		check range of string with digit
 * Arguments:		string, min, max
 * Return Value:	successful: 0, otherwise: -1
 */
int
checkRange(s, min, max)
	const String	s;
	int		min;
	int		max;
{
  int	val;

  if (checkDigit(s) == -1 || min > max) {
    return -1;
  }

  val = atoi(s);

  return (val >= min && val <= max) ? 0 : -1;
}

/*
 * Function Name:	checkIPAddr
 * Description:		check ip address
 * Arguments:		string
 * Return Value:	successful: 0, otherwise: -1
 */
int
checkIPAddr(s)
	const String	s;
{
  String	ipaddr;
  String	args[5];
  String *	ap;
  int		i;
  int		ret = 0;

  if (s == NULL || s[0] == EOL) {
    return -1;
  }

  ipaddr = strdup(s);
  if (ipaddr == NULL) {
    return -1;
  }

  for (i = 0; i < 5; i++) {
    args[i] = NULL;
  }

  for (i = 0, ap = args; (*ap = strsep(&ipaddr, ".")) != NULL; i++) {
    args[i] = *ap;
    if (**ap != EOL) {
      if (++ap >= &args[5]) {
	ret = -1;
	goto done;	/* error: arguments overflow */
      }
    }
  }

  if (i > 4 ||
      checkRange(args[0], 0, 255) == -1 ||
      checkRange(args[1], 0, 255) == -1 ||
      checkRange(args[2], 0, 255) == -1 ||
      checkRange(args[3], 0, 255) == -1) {
    ret = -1;
    goto done;
  }

done:
  free((String)ipaddr);

  return ret;
}

/*
 * Function Name:	checkNetAddr
 * Description:		check network address
 * Arguments:		string
 * Return Value:	successful: 0, otherwise: -1
 */
int
checkNetAddr(s)
	const String	s;
{
  String	netaddr;
  String	masklen;
  int		ret = 0;

  if (s == NULL || s[0] == EOL) {
    return -1;
  }

  netaddr = strdup(s);
  if (netaddr == NULL) {
    return -1;
  }

  masklen = (String)strrchr(netaddr, '/');
  if (masklen == NULL) {
    ret = -1;
    goto done;
  }
  *masklen++ = EOL;

  if (checkIPAddr(netaddr) == -1 || checkRange(masklen, 0, 32) == -1) {
    ret = -1;
    goto done;
  }

done:
  free((String)netaddr);

  return ret;
}

/*
 * Function Name:	checkIPandPort
 * Description:		check ip address and port pair
 * Arguments:		string
 * Return Value:	successful: 0, otherwise: -1
 */
int
checkIPandPort(s)
	const String	s;
{
  String	ipaddr;
  String	port;
  int		ret = 0;

  if (s == NULL || s[0] == EOL) {
    return -1;
  }

  ipaddr = strdup(s);
  if (ipaddr == NULL) {
    return -1;
  }

  port = (String)strrchr(ipaddr, ':');
  if (port == NULL) {
    ret = -1;
    goto done;
  }
  *port++ = EOL;

  if (checkIPAddr(ipaddr) == -1 || checkRange(port, 0, 65535) == -1) {
    ret = -1;
    goto done;
  }

done:
  free((String)ipaddr);

  return ret;
}


/*****************************************************************************
 *****************************************************************************/

#if	DEBUG_CHECK
int
main(argc, argv)
	int	argc;
	char *	argv[];
{
  if (argc == 1) {
    fprintf(stderr, "usage: check string\n");
    exit(1);
  }
  if (checkEmpty(*++argv) == 0) {
    fprintf(stderr, "%s: empty\n", *argv);
  }
  if (checkDigit(*argv) == 0) {
    fprintf(stderr, "%s: digit\n", *argv);
  }
  if (checkAlphabet(*argv) == 0) {
    fprintf(stderr, "%s: alphabet\n", *argv);
  }
  if (checkUserName(*argv) == 0) {
    fprintf(stderr, "%s: username\n", *argv);
  }
  if (checkHostName(*argv) == 0) {
    fprintf(stderr, "%s: hostname\n", *argv);
  }
  if (checkDomainName(*argv) == 0) {
    fprintf(stderr, "%s: domainname\n", *argv);
  }
  if (checkEMail(*argv) == 0) {
    fprintf(stderr, "%s: e-mail\n", *argv);
  }
  if (checkRange(*argv, 0, 255) == 0) {
    fprintf(stderr, "%s: range\n", *argv);
  }
  if (checkIPAddr(*argv) == 0) {
    fprintf(stderr, "%s: ipaddr\n", *argv);
  }
  if (checkNetAddr(*argv) == 0) {
    fprintf(stderr, "%s: netaddr\n", *argv);
  }
  if (checkIPandPort(*argv) == 0) {
    fprintf(stderr, "%s: ipaddr+port\n", *argv);
  }
  else {
    fprintf(stderr, "%s: unknown\n", *argv);
  }

  return 0;
}
#endif	/* DEBUG_CHECK */

