/*****************************************************************************
 *
 * FILE:	functions.h
 * DESCRIPTION:	functions declaration file for cockatrice sources
 * DATE:	Wed, Feb 14 2001
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.com/cover.html
 * COPYRIGHT:	(C) 2001 졿Kouichi ABE (WALL), All rights reserved.
 * $Id: functions.h,v 1.11 2002/07/07 10:08:04 kouichi Exp $
 *
 *****************************************************************************/

#ifndef	_functions_h
#define	_functions_h

#if	_cockatrice_h
/* cookie.c */
extern void		cookieInit(void);
extern void		cookieRelease(void);
extern int		cookieParse(void);
extern String		cookieGet(const String);
extern int		cookieSetName(const String, const String);
extern int		cookieSetComment(const String, const String);
extern int		cookieSetDomain(const String, const String);
extern int		cookieSetMaxAge(const String, unsigned int);
extern int		cookieSetPath(const String, const String);
extern int		cookieSetSecure(const String, bool_t);
extern void		sendResponseHeader(const String);


/* check.c */
extern int		checkEmpty(const String);
extern int		checkDigit(const String);
extern int		checkAlphabet(const String);
extern int		checkUserName(const String);
extern int		checkHostName(const String);
extern int		checkDomainName(const String);
extern int		checkEMail(const String);
extern int		checkRange(const String, int, int);
extern int		checkIPAddr(const String);
extern int		checkNetAddr(const String);
extern int		checkIPandPort(const String);


/* date.c */
extern void		dateInit(void);
extern String		getToday(void);
extern int		getYearOfToday(void);
extern int		getMonthOfToday(void);
extern int		getDayOfToday(void);
extern int		getWDayOfToday(void);
extern bool_t		isFestival(int, int, int);
extern int		getDayOfTheWeek(int, int, int);
extern String		getTime(void);


/* env.c */
extern int		envInit(void);
extern void		envRelease(void);
extern String		envServerSoftware(void);
extern String		envServerName(void);
extern String		envGatewayInterface(void);
extern String		envServerProtocol(void);
extern unsigned int	envServerPort(void);
extern String		envRequestMethod(void);
extern String		envScriptName(void);
extern String		envPathInfo(void);
extern String		envPathTranslated(void);
extern String		envRemoteHost(void);
extern String		envRemoteAddr(void);
extern unsigned int	envRemotePort(void);
extern String		envAuthType(void);
extern String		envRemoteUser(void);
extern String		envRemoteIdent(void);
extern String		envHttpAccept(void);
extern String		envHttpAcceptEncoding(void);
extern String		envHttpAcceptLanguage(void);
extern String		envHttpUserAgent(void);
extern String		envHttpReferer(void);
extern String		envDocumentRoot(void);
extern String		envServerRoot(void);
extern String		envServerAdmin(void);


/* lock.c */
extern int		lock(const String);
extern int		unlock(const String);
extern void		unlockAll(void);


/* param.c */
extern void		paramInit(void);
extern void		paramRelease(void);
extern int		paramPut(const String, const String);
extern String		paramGet(const String);
extern VList *		paramsGet(const String);


/* session.c */
extern int		sessionOpenDB(const String);
extern int		sessionCloseDB(void);
#if	0
extern int		sessionInit(void);
extern void		sessionDone(void);
extern int		sessionChangeDB(const String);
#endif
extern void		sessionSetExpires(unsigned int);
extern unsigned int	sessionGetExpires(void);
extern String		sessionGenerateID0(void);
extern String		sessionGenerateID1(const String);
extern String		sessionGenerateID2(const String, const String);
extern int		sessionValidateID(const String);
extern int		sessionDeleteID(const String);
extern int		sessionCountValidID(void);


/* upload.c */
extern int		parseMultiPartForm(void);
extern FILE *		openUploadFile(const String);
extern void		closeUploadFiles(void);
extern int		storeUploadFile(const String, const String);
extern String		digestUploadFile(const String, digest_t);


/* util.c */
extern String		chomp(String);
extern unsigned int	hash(String, size_t);
extern String		unescapeURI(String);
extern String		escapeURI(String);
extern String		digestString(const String, digest_t);
#ifndef	HAVE_MKSTEMP
extern int		mkstemp(String);
#endif	/* HAVE_MKSTEMP */

/* html.c */
extern HTML *		newHTML401(dtd_t);
extern void		html401Done(void);
#endif	/* _cockatrice_h */

#if	_basilisk_h
/* html20.c */
extern HTML *		newHTML20(void);

/* html32.c */
extern HTML *		newHTML32(void);

/* html401.c */
extern HTML *		newHTML401(dtd_t);
#endif	/* _basilisk_h */

#endif	/* _functions_h */
